package MobileMonitoring;

import StaticMonitoring.Settings;
import SOMA.agent.*;
import SOMA.naming.*;
import SOMA.utility.*;
import com.adventnet.snmp.snmp2.*;
import java.io.*;
import java.lang.*;
import java.util.*;
import java.net.*;
import java.awt.event.*;

//N.B. continuo ad utilizzare "current" per poter sfruttare le funzioni gia' definite nella superclasse
//e' come se avessi un itenerario fatto di un solo nodo

/**
*    Si occupa della ripetizione  di operazioni atomiche (GET/GETNEXT/GETBULK) su un solo nodo
*    puo' avere un intervallo di osservazione determinato oppure continuare il polling finche' non gli
*    arriva un messaggio di "STOP" dal manager che lo ha delegato
**/
public class PollerAtomicOpMA extends SnmpAtomicOpMA implements StaticMonitoring.Operations
{
	//intervallo di osservazione
	//rimane a 0 quando si vuole controllare da remoto la terminazione
	//(N.B. in questo caso deve essere creato come traceable)
	int observation = 0;
	//intervallo di polling
	int pollint = 0;
	//numero di ripetizione da effettuare: verra' utilizzato solo se si ha un intervallo di osservazione
	//predeterminato
	int nvolte = 0;
	transient Alarm timer = null;
	//questa var mi serve a sapere se l'Alarm e' stato fermato o si e' fermato 
	//perche' e' finito il pollint
	transient boolean expired = true;
	//visto che dovro' interagire con un solo nodo introduco come variabile di classe
	//la sessione che utilizzero', anche perche' sia visibile all'interno del metodo di gestione
	//dell'evento
	transient SnmpSession session = null;
  transient Vector curris = new Vector();
	//vettore in cui memorizzare tutti i risultati: ogni elemento rappresentera' un campione di polling
	Vector job_result = null;
	

	public void putArgument(Object o)
	{
		InfoPolling Info = (InfoPolling)o;
		observation = Info.HowLong();
		pollint = Info.HowOften();
		nvolte = (observation/pollint);
		InfoAtomicSnmpOp subInfo = (InfoAtomicSnmpOp)Info.getInfo();
		super.putArgument(subInfo);
		job_result = new Vector();
	}

	public void doLocalJob(Settings settings,SnmpSession sess)
	{
		PrintStream out = agentSystem.getOut();
    int N=nvolte;
	    session = sess;
	    Settings sets = settings;
	    //out.println("Inizializing polling for "+observation+" with gaps of "+pollint);
		
		timer = new Alarm(pollint);
		
		if (observation == 0) //caso in cui non e' precisato l'intervallo totale di osservazione
       	{
       		//creo un thread che sta in attesa dello stop sulla mailbox
       		//e quando arriva ferma l'Alarm
       		new Thread(){
       		public void run()
       		{ 
       			boolean next = true;
       			while(next)
       			{
        			Message answer = mailbox.getMessage();
        			String temp = (String)answer.message;
        			if (temp.equals("STOP"))
        			{
        				timer.stop();
        				next = false;
        			}
        			
				}
			}}.start();
		        
			timer.start();
			//finche' l'Alarm non viene bloccato per un messaggio di "STOP" ricevuto
			//(cioe' finche' muore naturalm.!) si continua il polling
			while(expired)
			{
				poll(sets);
				expired = timer.await(Alarm.FOREVER);
			}
			
			session.close();
			api.close();
		}
      	
        //N.B. qui potrei aggiungere il caso in cui si vogliano solo i dati fino ad ora raccolti
        else
        {
	       		timer.start();
      		//out.println("Timer started");
			for (int i=N; --N >= 0; )
			{
    			poll(sets);
				timer.await(Alarm.FOREVER);
			}
			timer.stop();
	   		session.close();
			api.close();
 		 	goHome();
        }
		        
	}

	void poll(final Settings sets)						
	{
		new Thread(){
		public void run(){
		if (expired) //e' stato fermato
		{
      job_result.add(current,curris);
			session.close();
			api.close();
			goHome();
		}
		else  //e' scaduto il pollint
		{
			SnmpPDU sample = null;
    		sample = snmpOp(sets,session,reqOIDs,op_snmp,argsop);
			if (check(sample) == false)
     			curris.add(null);
    	else
			{
       			Vector varbinds = sample.getVariableBindings();
       			SnmpVar[] localres = new SnmpVar[varbinds.size()];
    			int j=0;
        		//dal vettore di var bind ricavo le snmpvar e le copio nel vettore dei risultati
        		for (Enumeration e=varbinds.elements(); e.hasMoreElements(); j++)
           		localres[j] = ((SnmpVarBind)e.nextElement()).getVariable();
       			curris.add(localres);
			}
		}}
		}.start();
		
	}

}
