package MobileMonitoring;

import StaticMonitoring.Settings;
import SOMA.*;
import SOMA.agent.*;
import SOMA.naming.*;
import SOMA.utility.*;
import com.adventnet.snmp.snmp2.*;
import java.io.PrintStream;
import java.util.*;
import java.net.*;

public class SnmpMobileManager extends Agent
{
	//identificatore dell'agente che lo ha delegato
	AgentID delegator = null;
    //PlaceID di destinazione
    PlaceID[] targets = null;
    //porte d'ascolto degli Agenti Snmp sui Place da visitare
    int[] snmpPorts = null;
    //versioni supportate dagli Agenti Snmp da interrogare
    int[] snmpVs = null;
    //stato del debugging: se true si avra' visualizzazione di messaggi
    //sul PrintStream associato all'Environment corrente
    boolean debug = false;

    //cronometro per il calcolo del tempo impiegato dall'agente per svolgere
    //il suo lavoro
    Chronometer Crono = new Chronometer("Total Time");
    //thread corrente di controllo delle sessioni Snmp
    transient SnmpAPI api;
    //Place di partenza
    PlaceID home;
    //PlaceID corrispondente alla prossima tappa
    PlaceID nexthop;
    //tappa corrente (0 la prima, 1 la seconda..)
    int current;
    //nome dell'agente in esecuzione
    String identity;
    //numero di Place da visitare
    int Ntargets;
    //risultato dell'azione svolta dall'agente nel suo viaggio: in generale si avra' un elem. per ogni target
    Vector job_result = null;

    //vettore di variabili che mi indicano come si e' concluso il lavoro su ogni place
    //N.B. per ora metto true/false potrei pensare poi di memorizzare indicazioni
    //piu' articolate di cosa e' successo (es. associare un codice ad ogni evento indesiderato)
    boolean[] result_code;

    /**qui si recuperano le singole informazioni a partire dall'oggetto passato
    //in input e si inizializzano gli altri attributi.
    //o e' un oggetto della classe DrivePlan
    */
    public void putArgument(Object o)
    {
        DrivePlan Info = (DrivePlan) o;
        targets = Info.getDestinations();
        snmpPorts = Info.getPorts();
        snmpVs = Info.getVersions();
        debug = Info.isDebug();
        delegator = Info.getDelegator();
        current = 0;
        Ntargets = targets.length;
        job_result = new Vector(Ntargets);
        result_code = new boolean[Ntargets];
        identity = "\""+getID().toString()+"\"";
    }

    /**questo  il metodo di partenza, eseguito all'attivazione dell'agente:
    //qui si avvia il cronometro che conter il tempo necessario al completamento
    //dell'operazione, si definir la prima tappa del "viaggio" e si inizializzer
    //il campo home che dovr poi essere utilizzato per il ritorno al Place
    //di partenza
    */
    public void run()
    {
        PrintStream out = agentSystem.getOut();
        try
        {
            home = agentSystem.getPlaceID();
            nexthop = targets[current];
            //trace(out,getHeader()+this.getClass().getName()+" running");
            Crono.Start();
            goNext();
        }
        catch(Exception e) {e.printStackTrace();}
    }

    /**metodo di passaggio ad una tappa successiva,  qui che si controlla se il viaggio
    // finito, in tal caso si torna al Place di partenza
    */
    public void goNext()
    {
        PrintStream out = agentSystem.getOut();
//out.println("NEXT");

        if (current == Ntargets) //ha visitato tutti
            goHome();
        else
        try  {go(nexthop,"prepareLocalJob");}
        catch(Exception e){e.printStackTrace(out);}
    }

    /**metodo di preparazione dell'ambiente locale ai fini dell'adempimento
    //del compito: creazione thread di gestione delle sessioni Snmp, creazione
    //di una sessione Snmp per l'interazione con l'Agente Snmp presente localmente
    //non si prevede recupero di informazioni da configurazione
    */
    public void prepareLocalJob()
    {
        PrintStream out = agentSystem.getOut();

        //caso in cui sono finita su un place diverso da quello desiderato
        if (agentSystem.getPlaceID() != targets[current])
        {
            trace_err(out,getHeader()+" Wrong Target. Right is: "+targets[current]);
            job_result.add(current,null);
	        result_code[current] = false;
            if (++current != Ntargets)
      	      nexthop = targets[current];
            goNext();
        }

        api = new SnmpAPI();
        api.start();

        SnmpSession session = new SnmpSession(api);
        //setto i parametri locali: indirizzo (localhost) e porta (la prima libera assegnata dal sistema)
        //da cui inviare le richieste all'agente Snmp.. in realta' e' gia' cosi' il default
        //String[] addrs = new String[1];
        //addrs[0] = "127.0.0.1":
        //session.setLocalAddresses(addrs);
        //session.setLocalPort(0);
        session.setRemotePort(snmpPorts[current]);
        session.setPeername("127.0.0.1");
        int version = snmpVs[current];
        //se la versione e' la 3 assegno il default per lo username
        //negli altri casi il default e' gia' "public"
        if (version == SnmpAPI.SNMP_VERSION_3)
            session.setUserName((new String("initial")).getBytes());
        session.setVersion(snmpVs[current]);
        try {session.open();}
        catch(SnmpException ex) {trace_err(out,getHeader()+" Error opening the session");}

        doLocalJob(session);
    }

    //qui sono racchiuse le operazioni necessarie all'adempimento del compito
    //specifico di questo Agente Mobile
    public void doLocalJob(SnmpSession session)
    {
        String res = checkSys(session);
        job_result.add(current,res);
        result_code[current] = true;

        session.close();
        api.close();

		if (++current != Ntargets)
	        nexthop = targets[current];
        goNext();

    }

    String checkSys(SnmpSession session)
    {
        SnmpVar sys;
        try {sys = session.get(".1.3.6.1.2.1.1.1.0");}
        catch(SnmpException e) { return null;}
        if (sys != null)
            return ((String)sys.toValue());
        else return null;
    }

    //metodo di ritorno al Place di partenza
    public void goHome()
    {
    
        try{ go(home,"EndofPlan");}
        catch (Exception e) {
                PrintStream out = agentSystem.getOut();
				e.printStackTrace(out);}
    }

    //metodo che racchiude le azioni da svolgere una volta tornati al Place di partenza
    public Message EndofPlan()
    {
  	    Message ret = new Message(job_result,getID(),delegator);
        Environment env = agentSystem.getEnvironment();
        //trace(out,getHeader()+" SENDING INFO TO: "+delegator.toString());
	    env.agentManager.sendMessage(ret);
        PrintStream out = agentSystem.getOut();
        Crono.Stop();
        out.println(getHeader()+" JOB DONE IN "+Crono.toString());
        //ShowJob(out);

	    return ret;
    }

	public String getHeader()
	{
		String location = agentSystem.getPlaceID().toString();
		return ("@@@@@ "+identity+" on "+location+": ");
	}

    //metodo di visualizazione dei risultati su un particolare PrintStream
    void ShowJob(PrintStream o)
    {
        o.println(getHeader()+" Data retrieved from "+getID().toString());
        for (int j=0; j<Ntargets; j++)
            o.println(targets[j].toString()+":"+(String)job_result.elementAt(j));
    }

    //visualizza una stringa X sul PrintStream o se il debugging  abilitato
    void trace(PrintStream o, String X)
    {
        if (debug)
            o.println(X);
    }

    void trace_err(PrintStream o, String X)
    {
        if (debug)
        {
        	o.print(getClass().getName()+">");
            o.println(X);
         }
    }
}