package MonitorConfig;

import java.util.*;
import java.io.*;
import java.net.*;
import com.adventnet.snmp.snmp2.SnmpAPI;
import SOMA.naming.*;
import SOMA.naming.place.*;
import SOMA.*;

public class InfosAgentSnmp extends Debug implements Serializable
{
	//hashtable contenente per ogni coppia (host:porta) associata a un agente le informazioni:
	//versione che supporta, nome di sicurezza da utilizzare, place di appartenenza
    Hashtable infos = new Hashtable();
    Hashtable placetoport = new Hashtable();
    //introduco come attributo anche file di serializzazione prevedendo
    //di poter utilizzare un file diverso per ogni host
    File serFile = null;


	//N.B. ogni volta che introduco/cancello/modifico un'entry di infos aggiorno anche la relativa entry 
	//di placetoport in modo trasparente
	
	
    public InfosAgentSnmp(String file,boolean deb, PrintStream o)
    {
        super(deb,o);
        try {serFile = new File(file);}
        catch(NullPointerException ne) {trace_err("Attemp to create a file without its name"); return;}
    }


	boolean addEntry(IpAddress k,entry e)
    {
        if (infos.containsKey(k))
        {
            trace_err("Esiste gia' un'entry corrispondente a: "+k.toString());
            return false;
        }
        else
        try
        {
            infos.put(k,e);
        }
        catch (NullPointerException ex){ return false;}
        placetoport.put(e.place,new Integer(k.getPort()));
        return true;
    }

	/**aggiunge un agente Snmp all'elenco di quelli noti
	* host e' l'IP dell'host su cui si trova l'agente
	* port e' la porta d'ascolto dell'agente
	* version e' la versione supportata dall'agenteSnmp
	* sec_name e' il nome di sicurezza con cui ci si deve presentare all'agente
	* restituisce true se ha aggiunto con successo l'agente all'elenco di quelli noti
	*             false altrimenti
	*/
    public boolean addEntry(String host, int port, int version,String sec_name,PlaceID p)
    {
    	IpAddress k = new IpAddress(host,port);
        entry e = new entry(version,port,sec_name,p);
        return(addEntry(k,e));
    }

	/**aggiunge un agente Snmp all'elenco di quelli noti
	* host e' l'IP dell'host su cui si trova l'agente (la porta e' quella di default: 161)
	* version e' la versione supportata dall'agenteSnmp
	* sec_name e' il nome di sicurezza con cui ci si deve presentare all'agente
    * restituisce true se ha aggiunto con successo l'agente all'elenco di quelli noti
	*             false altrimenti

	*/
    public boolean addEntry(String host,int version,String sec_name,PlaceID p)
    {
    	IpAddress k = new IpAddress(host);
        entry e = new entry(version,sec_name,p);    	
        return(addEntry(k,e));
    }

	/**aggiunge un agente Snmp all'elenco di quelli noti
	* host e' l'IP dell'host su cui si trova l'agente (la porta e' quella di default: 161)
	* la versione supportata dall'agente Snmp e' quella di default: V1
	* il nome di sicurezza con cui ci si deve presentare all'agente e' quello di default: "public"
    * restituisce true se ha aggiunto con successo l'agente all'elenco di quelli noti
	*             false altrimenti

	*/
    public boolean addEntry(String host,PlaceID p)
    {
    	IpAddress k = new IpAddress(host);
        entry e = new entry(p);
        return(addEntry(k,e));
    }

    public boolean removeEntry(String host,int port)
    {
    	IpAddress k = new IpAddress(host,port);
    	PlaceID p = ((entry)infos.get(k)).place;
        if (infos.remove(k) == null)
        {
            trace_err("Non esiste un'entry corrispondente a "+host+":"+port);
            return false;
        }
        placetoport.remove(p);
        return true;
    }

    public void modifyEntry(IpAddress k,entry e)
    {
    	entry temp = (entry)infos.get(k);
        if (temp != null)
        {
        	placetoport.remove(temp.place);  
            infos.remove(k); 
        }   

       	placetoport.put(e.place,new Integer(161));
        infos.put(k,e);
        return;
    }

    public void modifyEntry(String host,int port, int version,String sec_name,PlaceID p)
    {
    	IpAddress k = new IpAddress(host,port);
        entry e = new entry(version,sec_name,p);
        modifyEntry(k,e);
    }

    public void modifyEntry(String host,int version,String sec_name,PlaceID p)
    {
    	IpAddress k = new IpAddress(host);
        entry e = new entry(version,sec_name,p);
        modifyEntry(k,e);
    }

    public void modifyEntry(String host,PlaceID p)
    {
    	IpAddress k = new IpAddress(host);
        entry e = new entry(p);
        modifyEntry(k,e);
    }

    public void modifyEntry(String host,int port,PlaceID p)
    {
    	IpAddress k = new IpAddress(host,port);
        entry e = new entry(p);
        modifyEntry(k,e);
    }

	public int getSupportedVersion(String host)
    {
		return(getSupportedVersion(host,161));
    }

    public int getSupportedVersion(String host,int port)
    {
    	IpAddress k = new IpAddress(host,port);
        entry e = (entry)infos.get(k);
        if (e == null)
        	return -1;
	   	return (e.getVersion());
    }

    public String getSecurityName(String host,int port)
    {
    	IpAddress k = new IpAddress(host,port);
        entry e = (entry)infos.get(k);
        if (e == null)
        	return null;
        return(e.getSecName());
    }

    public String getSecurityName(String host)
    {
        return(getSecurityName(host,161));
    }

    public PlaceID getCorrespondingPlace(String host,int port)
    {
    	IpAddress k = new IpAddress(host,port);
        entry e = (entry)infos.get(k);
        if (e == null)
        	return null;
        return(e.getPlace());
    }

    public PlaceID getCorrespondingPlace(String host)
    {
    	return(getCorrespondingPlace(host,161));
    }
    
    /**restituisce le porte d'ascolto degli agenti SNMP presenti sui place specificati, 
    //purche' questi siano stati configurati
    //restituisce null se tra i place ce n'e' almeno 1 non configurato
    */
    public int[] getCorrespondingPorts(PlaceID places[])
    {
    	int[] corrisp = new int[places.length];
    	for (int j=0; j<places.length; j++)
    		if (placetoport.containsKey(places[j]))
	    		corrisp[j] = ((Integer)placetoport.get(places[j])).intValue();
	    	else
	    	{
	    		trace_err("Agent SNMP on place "+places[j]+" hasn't been configured"); 
	    		return null;
	    	}
	    		
    	return corrisp;
    }
    				

    public boolean setSerializeFileName(String file)
    {
		try{ serFile = new File(file); }
		catch(NullPointerException ex) {trace_err("Attempt to set serialize file without its name"); return false;}
        return true;
    }

    public String getSerializeFileName()
    {
        return (serFile.getAbsolutePath());
    }

    public void showContent()
    {
    	out.println("(Host:SnmpPort)  SecName  SnmpVersion   Place");
    	for (Enumeration e = infos.keys(); e.hasMoreElements();)
    	{
    		IpAddress curkey = (IpAddress)e.nextElement();
    		out.print(curkey.toString()+" ");
    		entry cur = (entry)infos.get(curkey);
    		cur.show(out);
	   	}
    }

    public void serialize()
    {
        try
        {
            FileOutputStream fout = new FileOutputStream(serFile);
            ObjectOutputStream out = new ObjectOutputStream(fout);
            out.writeObject(this);
        }
        catch(Exception e) { }

    }

    public boolean deSerialize()
    {
        try
        {
        	//se il file esiste ed e' leggibile
        	if (serFile.canRead())
	        try
    	    {
        	    FileInputStream fin = new FileInputStream(serFile);
            	ObjectInputStream in = new ObjectInputStream(fin);
           	    InfosAgentSnmp copy = (InfosAgentSnmp) in.readObject();
            	infos = copy.infos;
            	//in realta' questi due campi gia' dovrebbero coincidere
		        serFile = copy.serFile;
	            return true;
    	    }
        	catch(Exception e) {trace_err("Problems occurred during deserialization");}
        }
        catch(SecurityException ex)
        {
        	String abs = serFile.getAbsolutePath();
        	trace_err("Permission to read deserialization file "+abs+" denied on this place");
        }

        return false;
    }

    class entry implements Serializable
    {
        int snmp_version;
        String sec_name;
		PlaceID place;

        entry(PlaceID p)
        {
            snmp_version = SnmpAPI.SNMP_VERSION_1;
            sec_name = "public";
            place = p;
        }

        entry(int version, String name, PlaceID p)
        {
            snmp_version = version;
            sec_name = name;
            place = p;
        }

        entry(int version, int port, String name, PlaceID p)
        {
            snmp_version = version;
            sec_name = name;
            place = p;
        }

        public int getVersion()
        {
            return snmp_version;
        }

        public String getSecName()
        {
            return sec_name;
        }

        public PlaceID getPlace()
        {
        	return place;
        }

		public void show(PrintStream out)
        {
        	String version = null;

        	if (snmp_version == SnmpAPI.SNMP_VERSION_1)
        		version = "v1";
        	else if (snmp_version == SnmpAPI.SNMP_VERSION_2C)
        		version = "v2C";
        	else if (snmp_version == SnmpAPI.SNMP_VERSION_3)
        		version = "v3";
        	out.println(sec_name+"	 "+version+"   ("+place.toString()+")");
        }
    }

}