package StaticMonitoring;

import MonitorConfig.IpAddress;
import com.adventnet.snmp.snmp2.SnmpVar;
import com.adventnet.snmp.snmp2.SnmpAPI;
import java.util.*;
import java.io.*;

public class InfoDomain implements Serializable
{
	//vettore contenente per ogni variabile le informazioni sul valore max
	RefVal[] max;
	//vettore contenente per ogni variabile le informazioni sul valore min
	RefVal[] min;
	//vettore contenente per ogni variabile le informazioni sul valore medio
	ElemAve[] ave;
	//numero di variabili coinvolte
	int Nids;

	public InfoDomain(int Nvar)
	{
		Nids = Nvar;
		max = new RefVal[Nvar];
		min = new RefVal[Nvar];
		ave = new ElemAve[Nvar];
		//inizializzo le strutture di max/min/media
		for (int i=0; i<Nids; i++)
		{
			max[i] = new RefVal();
			min[i] = new RefVal();
			ave[i] = new ElemAve();
		}
	}


	public Object getMax(int index)
	{
		return max[index].getCurRefVal();
	}

	public Object getMin(int index)
	{
		return min[index].getCurRefVal();
	}

	public IpAddress[] getMinLocations(int index)
	{
		return min[index].getWhere();
	}

	public IpAddress[] getMaxLocations(int index)
	{
		return max[index].getWhere();
	}

	public Object getAve(int index)
	{
		return ave[index].getAve();
	}

	//aggiorna il valore min/max di una certa variabile
	public Object setCurMinMax(SnmpVar currel,int index, String cond, IpAddress curloc)
	{
		RefVal rif;
		if (cond.equals(">"))
			rif = max[index];
		else
			rif = min[index];

		return(rif.upDate(currel,cond,curloc));
	}

	//aggiorna il valore somma di una certa variabile
	public Object setCurSum(SnmpVar currel,int index)
	{
		return (ave[index].Sum(currel));
	}

	//calcola il valore medio di ogni variabile con riferimento al numero di agenti
	//Snmp da cui si e' reperito un valore
	public void setAve()
	{
		for (int j=0; j<Nids; j++)
			ave[j].calcAve();
    }

  	public void show(PrintStream out,int i)
  	{
  		Object temp;
  		IpAddress[] where;
  		
  		temp = min[i].value;
  		if (temp != null)
  		{
  			out.println("MIN: "+temp+" at:");
  			where = min[i].getWhere();
  			if (where != null)
  			for (int j=0; j<where.length; j++)
  				out.println(where[j].toString());
  		}
		temp = max[i].value;
		if (temp != null)
		{
  			out.println("MAX: "+temp+" at:");
  			where = max[i].getWhere();
  			if (where != null)
  			for (int j=0; j<where.length; j++)
  				out.println(where[j].toString());
  		}
		
		temp = ave[i].getAve();
		if (temp != null)
		{
  			out.println("AVERAGE calculated on  "+ave[i].getNtargets()+" :"+temp);
			out.println();
		}
  	}

	class RefVal implements Serializable
	{
		//valore di riferimento
		Object value = null;
		//vettore di (Ip:porta) corrispondente agli agenti in cui si e' rilevato val
		Vector where_is_val = new Vector();

		RefVal()
		{
		}

		RefVal(Object val,IpAddress where)
		{
			value = val;
			where_is_val.add(where);
		}

		void addWhere(IpAddress location)
		{
			where_is_val.add(location);
		}

		IpAddress[] getWhere()
		{
			return ((IpAddress[])where_is_val.toArray(new IpAddress[0]));
		}

		Object getCurRefVal()
		{
			return value;
		}

		void setNewRef(Object newval, IpAddress newloc)
		{
			value = newval;
			where_is_val.removeAllElements();
			where_is_val.add(newloc);
		}

		/**aggiorna le informazioni sul valore min/max di una variabile
		  *currel e' la SnmpVar corrispondente alla nuova istanza della variabile
		  *index e' l'indice della variabile nella lista di variabili considerate
		  *cond puo' essere "<" o ">" a seconda che si sia interessati al massimo o al minimo
		  *curloc e' l'IpAddress (host:port) da cui e' stato reperito il valore corrente della variabile
		  *se l'istanza corrente ha un valore minore/maggiore del min/max corrente, aggiorna il valore
		  *min/max e cancella le informazioni associate al precedente; in caso di valore uguale si aggiornano
		  *solo le informazioni sulle locazioni che assumono il massimo
		  *restituisce il valore min/max aggiornato che, a seconda del tipo di variabile potra' essere un
		  *Integer o un Long
		*/
		Object upDate(SnmpVar currel,String cond, IpAddress curloc)
		{
			byte eltype = currel.getType();
     	  	int ret;
       		Object curval;

      	 	if (eltype == SnmpAPI.INTEGER)
	  		{
	   		    curval = (Integer)currel.toValue();
	   		    if (value == null)
	  			{
	  				value = curval;
	  				where_is_val.add(curloc);
	  				return curval;
	  			}
				else
	    	    	ret = ((Integer)curval).compareTo((Integer)value);
      	  	}
       		else if ((eltype == SnmpAPI.COUNTER) || (eltype == SnmpAPI.COUNTER64) ||
           			(eltype == SnmpAPI.GAUGE) || (eltype == SnmpAPI.TIMETICKS))
	  		{
          	    curval = (Long)currel.toValue();
          	    if (value == null)
	  			{
	  				value = curval;
	  				where_is_val.add(curloc);
	  				return curval;
	  			}
				else
       	 	    	ret = ((Long)curval).compareTo((Long)value);
       		}
			else
				return value;

			if ( (ret > 0) && (cond.equals(">")) )
			{
				setNewRef(curval,curloc);
				return curval;
			}
			else if ( (ret < 0) && (cond.equals("<")) )
			{
				setNewRef(curval,curloc);
				return curval;
			}
			else if  (ret == 0)
				addWhere(curloc);

			return value;
		}

	}

	class ElemAve implements Serializable
	{
		byte eltype;
		//var da incrementare ogni volta che viene invocato il metodo Sum
		//questo permette di sapere su quanti agenti effettivamente viene fatta
		//la media
	    int Ntargets;
		Object value;

		ElemAve()
		{
			Ntargets = 0;
			value = null;
		}

		ElemAve(Object val, byte type)
		{
			Ntargets = 1;
			value = val;
			eltype = type;
		}

		Object getAve()
		{
			return value;
		}

		int getNtargets()
		{
			return Ntargets;
		}

		//aggiunge alla somma corrente dei valori, associati alla variabile individuata da
		//index, il valore di un'altra istanza della variabile;
		//currel e' l'elemento corrente
		//index e' l'indice (0..Ntargets) della variabile
		//restituisce la somma aggiornata
		// (N.B. a seconda del tipo della variabile si trattera' di Integer o Long)
		Object Sum(SnmpVar currel)
		{
			byte eltype = currel.getType();
 	      	if (eltype == SnmpAPI.INTEGER)
 	        {
		   	    Integer curval1 = (Integer)currel.toValue();
 			    if (value == null)
	  			{
	  				value = curval1;
	  				eltype = SnmpAPI.INTEGER;
 		          	Ntargets++;
	  				return value;
	  			}

 	          	value = new Integer(curval1.intValue()+((Integer)value).intValue());
 	          	Ntargets++;
 	 	    }
  	     	else if ((eltype == SnmpAPI.COUNTER) || (eltype == SnmpAPI.COUNTER64) ||
  	         	(eltype == SnmpAPI.GAUGE) || (eltype == SnmpAPI.TIMETICKS))
  	        {
   	      		Long curval2 = (Long)currel.toValue();
     		    if (value == null)
	  			{
	  				eltype = SnmpAPI.INTEGER;
	  				value = curval2;
       	    		Ntargets++;
		  			return value;
	  			}

       	    	value = new Long(curval2.longValue()+((Long)value).longValue());
       	    	Ntargets++;
       		}
       		return value;
    	}

    	Object calcAve()
    	{
    		if (Ntargets != 0)
    		{
				if (eltype == SnmpAPI.INTEGER)
					value = new Integer(((Integer)value).intValue()/Ntargets );
				else if ( (eltype == SnmpAPI.COUNTER) || (eltype == SnmpAPI.COUNTER64) ||
        		    (eltype == SnmpAPI.GAUGE) || (eltype == SnmpAPI.TIMETICKS) )
      			value = new Long(((Long)value).longValue()/Ntargets);
      			return value;
      		}
      		else
      			return null;
    	}

    }



}
