package StaticMonitoring;

import java.util.*;
import java.io.*;
import MonitorConfig.Debug;
import MonitorConfig.Configuration;
import com.adventnet.snmp.snmp2.*;

public class Settings extends Debug
{
    //varie variabili di supporto per trattare i parametri passati in input: determinano la posizione
    //che i rispettivi parametri occuperanno in un vettore
    static public final int PORT = 0;
    static public final int SECNAME = 1;
    static public final int VERSION = 2;
    static public final int AUTHPROTO = 3;
    static public final int AUTHPWD = 4;
    static public final int PRIVPROTO = 5;
    static public final int PRIVPWD = 6;
    static public final int CONTEXTNAME = 7;
    static public final int CONTEXTID = 8;
    static public final int TIMEOUT = 9;
    static public final int RETRIES = 10;
    static public final int NMAXOPTIONS = 11;

    static String options[] = {"-p","-cu","-v","-a","-aw","-pp","-pw","-cx","-cid","-t","-r"};


    //indirizzo IP del nodo su cui si trova l'agente SNMP remoto
    //public String remaddr = null;
    //public int remport = 161;
    //community cui appartiene l'utente in caso di richiesta conforme a SnmpV1/SnmpV2c
    //public String community = new String("public");
    //nome del Principal da usare in una richiesta conforme a SnmpV3
    public String userName = new String("");
    //protocollo di autenticazione con cui un agente SnmpV3 dovra' autenticare il Principal
    public int authProto = USMUserEntry.NO_AUTH;
    //password di autenticazione del Principal
    public String authPwd = new String("");
    //protocollo di privacy con cui un agente SnmpV3 dovra' decrittare le informazioni inviate dal Principal
    //public int privProto = USMUserEntry.CBC_DES;
    //password di privacy del Principal
    public String privPwd = new String("");
    //nome del contesto da cui parte la richiesta SnmpV3
    public String contextName = new String("");
    //nome del motore SnmpV3 che ha generato la richiesta
    public String contextID = new String("");
    //versione supportata dall'agente SNMP (quindi utilizzata nelle richieste per avere compatibilita')
    //public int version = SnmpAPI.SNMP_VERSION_1;
    //variabile che mi indica se per i settaggi si e' fatto ricorso alla configurazione
    public boolean config = false;
    //variabile che mi indica se i settaggi hanno avuto successo
    boolean success = false;
    //API di riferimento per il reperimento dei parametri dalla configurazione
    //SnmpAPI api_ref = null;

    public Settings(boolean deb,PrintStream o)
    {
       super(deb,o);
    }

    //costruttore che avvia il settaggio dei parametri di sessione, in parte ricevuti come argomento,
    //i rimanenti recuperati dalla configurazione
    //session e' la sessione gia' creata sulla quale settare i parametri
    //params e' il vettore dei parametri: la posizione di ogni parametro e' definita dalle variabili
    //static della classe
    //api e' il thread che monitora la sessione e possiede le informazioni di configurazione
    //per SnmpV3
    //deb mi dice se produrre output
    //o e' il PrintStream da utilizzare in caso di debug = true
   /* public Settings(SnmpSession session,String host, String params[], SnmpAPI api, boolean deb, PrintStream o)
    {
        super(deb,o);
        api_ref = api;
        config = true;
 
        if (setParamsConfig(session,host,params))
            success = true;
    }*/

    //costruttore che avvia il settaggio dei parametri specificati, nella sessione ricevuta come argomento
    //session e' la sessione gia' creata sulla quale settare i parametri
    //params e' il vettore dei parametri: la posizione di ogni parametro e' definita dalle variabili
    //static della classe
    //deb mi dice se produrre output
    //o e' il PrintStream da utilizzare in caso di debug = true
   /* public Settings(SnmpSession session,String host, String params[], boolean deb, PrintStream o)
    {
        super(deb,o);
        if (setParams(session,host,params))
            success = true;
    }*/

    public Settings(SnmpSession session,String host, String params[], boolean from_config, boolean deb, PrintStream o)
    {
        super(deb,o);
        config = from_config;
 
 		if (config) 
        	success = setParamsConfig(session,host,params);
        else 
			success = setParams(session,host,params);
    }
    //mi dice se tutti i settaggi sono andati a buon fine
    public boolean isSuccess()
    {
        return success;
    }

    //Con i metodi successivi setto i parametri di una sessione gia' creata, cosi' da renderla conforme
    //alla richiesta fatta (cioe' ai parametri dati in input)
    //params[] e' un vettore avente una posizione prefissata per ogni possibile parametro (vedi
    //variabili static di classe), sotto forma di stringhe, la' dove non e' stato specificato
    //il parametro corrispondente si ha null

    /**setta i parametri di una sessione gia' creata secondo quanto specificato nel vettore,
 	  *se ad un parametro corrisponde il valore null si assume il default
 	  *setta anche le variabili di classe corrispondenti
 	  *params e' il vettore dei parametri, ha una posizione prefissata per ogni possibile parametro di sessione
      *restituisce true se e' riuscito a settare tutti parametri specificati
      *            false altrimenti
      */
    boolean setParams(SnmpSession session,String host, String params[])
    {
        if (host == null)
        {
            trace_err("Usage error: Hostname is a mandatory argument");
            return false;
        }
        else
            session.setPeername(host);

        try
        {
            if (params[PORT] != null)
                session.setRemotePort(Integer.parseInt(params[PORT]));
            if (params[RETRIES] != null)
                session.setRetries(Integer.parseInt(params[RETRIES]));
            if (params[TIMEOUT] != null)
                session.setTimeout(Integer.parseInt(params[TIMEOUT]));
        }
        catch (NumberFormatException exc)
        {
            trace_err("Invalid Integer Argument");
            return false;
        }

        if (params[VERSION] != null)
        {
            if (params[VERSION].equals("v2"))
                session.setVersion(SnmpAPI.SNMP_VERSION_2C);
            else if (params[VERSION].equals("v1"))
                session.setVersion(SnmpAPI.SNMP_VERSION_1);
            else if (params[VERSION].equals("v3"))
                session.setVersion(SnmpAPI.SNMP_VERSION_3);
            else
            {
                trace_err("Invalid Version Number: "+params[VERSION]);
                return false;
            }
        }

        if (session.getVersion() == SnmpAPI.SNMP_VERSION_3)
        {
            if (params[SECNAME] != null)
            {
                userName = params[SECNAME];
                if ((params[AUTHPROTO] != null) && (params[AUTHPWD] != null))
                {
                    if (params[AUTHPROTO].equals("SHA"))
                        authProto = USMUserEntry.SHA_AUTH;
                    else if (params[AUTHPROTO].equals("MD5"))
                        authProto = USMUserEntry.MD5_AUTH;
                    else
                    {
                        trace_err("AuthPwd requires to specify a valid AuthProtocol");
                        return false;
                    }
                    authPwd = params[AUTHPWD];
                    
                    if (params[PRIVPWD] != null)
                        privPwd = params[PRIVPWD];

                }
                //caso in cui e' dato solo uno dei due parametri di autenticazione
                else if ( ((params[AUTHPROTO] == null)&& (params[AUTHPWD] != null))
                         ||  ((params[AUTHPROTO] != null)&& (params[AUTHPWD] == null)) )
                {
                    trace_err("Authentication requires to specify protocol & password");
                    return false;
                }
                //caso in cui non c'e' autenticazione ma e' stata specificata una password di privacy
                else if (params[PRIVPWD] != null)
                {
                    trace_err("Privacy needs authentication.Possible security levels: no auth, auth, auth&priv");
                    return false;
                }
                if (params[CONTEXTNAME] != null)
                    contextName = params[CONTEXTNAME];
                if (params[CONTEXTID] != null)
                    contextID = params[CONTEXTID];
            }
            else //secname == null
            {
                trace_err("Snmpv3 needs a UserName");
                return false;
            }

        }
        //caso in cui la versione non e' 3
        else if (params[SECNAME] != null)
            session.setCommunity(params[SECNAME]);

        return true;
    }


    /*setta i parametri di una sessione gia' creata a partire dal vettore params di
     *argomenti e da informazioni recuperate dalla configurazione locale (quelle sulla sicurezza)
     *vale solo per richieste Snmpv3
     *restituisce true se e' riuscito a settare i parametri specificati e a recuperare dalla
     *                 configurazione gli altri
     *            false altrimenti
     */
    boolean setParamsConfig(SnmpSession session,String host,String params[])
    {
    
  /* USMUserTable usm = api_ref.getUSMTable();
   SnmpEngineTable eng = api_ref.getSnmpEngine();
   
   if (usm != null)
   {
   for (Enumeration e= usm.getEnumeration();e.hasMoreElements();)
   {
   		USMUserEntry en = (USMUserEntry)e.nextElement();
   		byte[] ak = en.getAuthKey();
   		if (ak!=null)
	   		System.out.println("ak "+new String(ak));
	   	byte[] ap = en.getAuthPassword();
	   	if (ap != null)
	   		System.out.println("ap "+new String(ap));
   		System.out.println("proto"+en.getAuthProtocol());
   		byte[] engine = en.getEngineID();
   		if (engine != null)
	   		System.out.println("engine "+new String(engine));
   		byte[] user = en.getUserName();
   		if (user != null)
	   		System.out.println("user "+new String(user));
   		System.out.println();
   	}
   	}
   	
   	if (eng != null)
   {
   for (Enumeration e= eng.getEnumeration();e.hasMoreElements();)
   {
   		SnmpEngineEntry en = (SnmpEngineEntry)e.nextElement();
   		int eb = en.getEngineBoots();
   		System.out.println("time "+en.getEngineTime());
   		System.out.println("boot "+eb);
	   	byte[] eid = en.getEngineID();
	   	if (eid != null)
	   		System.out.println("eid "+new String(eid));
   		System.out.println("host "+en.getRemoteHost());
   		System.out.println("port "+en.getRemotePort());
   		System.out.println();
   	}
   	}*/
   	
        session.setVersion(SnmpAPI.SNMP_VERSION_3);
        if (host != null)
            session.setPeername(host);
        else
        {
            trace_err("Usage error: Hostname is a mandatory argument");
            return false;
        }
        try
        {
            if (params[PORT] != null)
                session.setRemotePort(Integer.parseInt(params[PORT]));
            if (params[RETRIES] != null)
                session.setRetries(Integer.parseInt(params[RETRIES]));
            if (params[TIMEOUT] != null)
                session.setTimeout(Integer.parseInt(params[TIMEOUT]));
        }
        catch(NumberFormatException e)
        {
            trace_err("Invalid Integer Argument");
            return false;
        }

  /*      SnmpEngineEntry engineEntry = api_ref.getSnmpEngine().getEntry(session.getPeername(),session.getRemotePort());
        if (engineEntry == null)
        {
            trace_err("Snmp Agent ("+session.getPeername()+":"+session.getRemotePort()+") not present in local configuration");
            return false;
        }
        byte[] engineID = engineEntry.getEngineID();*/
        if (params[SECNAME] == null)
        {
            trace_err("Snmpv3 needs username");
            return false;
        }
        userName = params[SECNAME];
/* System.out.println("user: "+userName+" engine: "+new String(engineID));
        USMUserEntry userEntry = api_ref.getUSMTable().getEntry((userName.getBytes()), engineID);
        if (userEntry == null)
        {
            trace_err("Association ("+userName+","+session.getPeername()+":"+session.getRemotePort()+") not present");
            return false;
        }
        if ((userEntry.getSecurityLevel()) != Snmp3Message.NO_AUTH_NO_PRIV)
        {
            authProto = userEntry.getAuthProtocol();
            authPwd = new String(userEntry.getAuthPassword());
 System.out.println("authpwd: "+authPwd);
 System.out.println("authkey: "+new String(userEntry.getAuthKey()));
            if ((userEntry.getSecurityLevel()) == Snmp3Message.AUTH_PRIV)
            //{
              //  privProto = userEntry.getPrivProtocol();
                privPwd = new String(userEntry .getPrivPassword());
            //}
        }*/

        if (params[CONTEXTNAME] != null)
            contextName = params[CONTEXTNAME];
        if (params[CONTEXTID] != null)
            contextID = params[CONTEXTID];

        //session.setUserName(userName.getBytes());
        //session.setPeername(remaddr);
        //session.setVersion(version);

        return true;
    }

    //processa il vettore degli argomenti, sotto forma di stringhe, nell'ordine:
    //["-opzione1" "valore opzione1"] ... ["-opzionek" "valore opzionek"] "OID" ["OID"..]
    //avendo come riferimento il vettore delle possibili opzioni, memorizzate come stringhe
    //"-opzione" nell'ordine in cui devono comparire in input
    //utile per un'interfaccia a linea di comando
    //restituisce il vettore degli OID e riempie un vettore passato come argomento, values, con i valori
    //corrispondenti alle opzioni, null per un opzione non specificata
    //input vettore degli argomenti, sotto forma di stringhe, nell'ordine:
    //["-opzione1" "valore opzione1"] ... ["-opzionek" "valore opzionek"] "OID" ["OID"..]
    //options il vettore delle possibili opzioni, memorizzate come stringhe "-opzione"
    //nell'ordine in cui devono comparire in input
    //values vettore in cui verranno memorizzati i valori corrispondenti ad ogni opzione, null
    //se non specificata
    //usage stringa da scrivere in caso di errore di input
    public String[] parseArguments(String input[],String options[],String values[],String usage)
    {
        boolean match = false;
        int i,j;
        i=j=0;

        //ora considero tutti gli arg. rimanenti: dovrebbe essere una successione di <opzione>
        //<valore> seguita poi da argomenti non corrispondenti ad opzioni (gli OIDs)
        for (; i<input.length; i++)
        {
            if (input[i].charAt(0) == '-')
            {
                //cerco l'opzione a cui corrisponde
                //visto che le opzioni sono memo. in options nello stesso ordine
                //con cui vengono passate in input, parto sempre dall'ultima posizione
                //raggiunta in options
                for( ; ((j<options.length) && (match==false)); j++)
                {
                    //se l'ho trovata devo leggerne il valore corrispondente
                    if (options[j].equals(input[i]))
                    {
                        //se c'e' subito un'altra opzione e' sbagliato, manca il valore
                        //dell'opzione precedente
                        if (input[++i].charAt(0) == '-')
                        {
                            trace_err("2"+usage);
                            return null;
                        }
                        values[j] = input[i];
                        match = true;
                    }
                    else //non ho trovato il valore corrispondente all'opzione corrente
                        values[j] = null;
                }
                if ((j == options.length)&& (match==false)) //non ho trovato un'opzione che matchi
                {
                    trace_err("3 "+input[i]+" "+usage);
                    return null;
                }
                match = false;
            }
            else //non si tratta di un opzione
                if (j == options.length) //ho gia' considerato tutte le possibili opzioni
                    break;  //esco dal for
                else
                {
                    for (int k=j+1; k<options.length; k++)
                        values[k] = null;
                    break;  //esco dal for
                }
        }
        if (input.length == i) //non sono rimasti arg. => no OID
            return null;
        String oids[] = new String[input.length-i];
        for (int k=0; i<input.length; i++,k++)
            oids[k] = input[i];

        return oids;
    }

	public void showParams(String[] args)
	{
		if (args.length != NMAXOPTIONS)
			return;
		for (int j=0; j<args.length; j++)
			trace(options[j]+" : "+args[j]);
	}

}